import * as path from "node:path";
import { addScript } from "#sitegen";
import { PrecomputedBlurhash } from "./blurhash.tsx";
import "./Video.css";
export namespace Video {
  export interface Props {
    title: string;
    width: number;
    height: number;
    sources: string[];
    downloads: string[];
    poster?: string;
    posterHash?: string;
    borderless?: boolean;
  }
}
 function PrecomputedBlurhash({ hash }: { hash: string }) {
export function Video(
  { title, sources, height, poster, posterHash, width, borderless }:
    Video.Props,
) {
  addScript("./video.client.ts");
  return (
    <figure class={`video ${borderless ? "borderless" : ""}`}>
      <figcaption>{title}</figcaption>
      {/* posterHash && <PrecomputedBlurhash hash={posterHash} /> */}
      {poster && <img src={poster} alt="waterfalls" />}
      <video
        controls
        preload="none"
        style={`width:100%;background:transparent;aspect-ratio:${
          simplifyFraction(width, height)
        }`}
        poster="data:null"
      >
        {sources.map((src) => (
          <source
            src={src}
            type={contentTypeFromExt(src)}
          />
        ))}
      </video>
    </figure>
  );
}
export function contentTypeFromExt(src: string) {
  if (src.endsWith(".m3u8")) return "application/x-mpegURL";
  if (src.endsWith(".webm")) return "video/webm";
  if (src.endsWith(".mp4")) return "video/mp4";
  if (src.endsWith(".ogg")) return "video/ogg";
  throw new Error("Unknown video extension: " + path.extname(src));
}
const gcd = (a: number, b: number): number => b ? gcd(b, a % b) : a;
function simplifyFraction(n: number, d: number) {
  const divisor = gcd(n, d);
  return `${n / divisor}/${d / divisor}`;
}
